@@ -1,12 +1,38 @@
-#!/usr/bin/env python2
+# -*- coding: utf-8 -*-
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Utility classes and functions for getmail.
 '''
 
+
+import os
+import os.path
+import socket
+import signal
+import stat
+import time
+import glob
+import re
+import fcntl
+import pwd
+import grp
+import getpass
+import subprocess
+import sys
+try:
+    import secrets
+except ImportError: #Py 2.7
+    class secrets:
+        @staticmethod
+        def token_hex(x):
+            return ''.join(['%02x' % ord(char)
+                     for char in open('/dev/urandom').read(x)])
+
 __all__ = [
     'address_no_brackets',
     'change_usergroup',
     'change_uidgid',
-    'decode_crappy_text',
     'format_header',
     'check_ssl_key_and_cert',
     'check_ca_certs',
@@ -27,33 +53,17 @@ __all__ = [
     'uid_of_user',
     'updatefile',
     'get_password',
-    'run_command',
+    'tostr',
+    'unicode',
 ]
 
-
-import os
-import os.path
-import socket
-import signal
-import stat
-import time
-import glob
-import re
-import fcntl
-import pwd
-import grp
-import getpass
-import commands
-import sys
-import tempfile
-import errno
-try:
-    import subprocess
-except ImportError, o:
-    subprocess = None
-
-# hashlib only present in python2.5, ssl in python2.6; used together
-# in SSL functionality below
+if sys.version_info.major > 2:
+    unicode = str
+    tostr = lambda lts: lts.decode()
+else:
+    tostr = lambda lts: lts
+    # due to above unicode entry in __all__
+    unicode = unicode  #noqa: PLW0127
 try:
     import ssl
 except ImportError:
@@ -71,6 +81,11 @@ try:
         gnomekeyring = None
 except ImportError:
     gnomekeyring = None
+# Optional Python keyring integration
+try:
+    import keyring
+except ImportError:
+    keyring = None
 
 from getmailcore.exceptions import *
 
@@ -107,7 +122,7 @@ def unlock_file(file, locktype):
         fcntl.flock(file, fcntl.LOCK_UN)
 
 #######################################
-def safe_open(path, mode, permissions=0600):
+def safe_open(path, mode, permissions=0o600):
     '''Open a file path safely.
     '''
     if os.name != 'posix':
@@ -115,7 +130,7 @@ def safe_open(path, mode, permissions=0600):
     try:
         fd = os.open(path, os.O_RDWR | os.O_CREAT | os.O_EXCL, permissions)
         file = os.fdopen(fd, mode)
-    except OSError, o:
+    except OSError as o:
         raise getmailDeliveryError('failure opening %s (%s)' % (path, o))
     return file
 
@@ -140,8 +155,8 @@ class updatefile(object):
             filename = os.path.join(os.path.dirname(filename),
                                     os.readlink(filename))
         try:
-            f = safe_open(self.tmpname, 'wb')
-        except IOError, (code, msg):
+            f = safe_open(self.tmpname, 'w')
+        except IOError as msg:
             raise IOError('%s, opening output file "%s"' % (msg, self.tmpname))
         self.file = f
         self.write = f.write
@@ -175,8 +190,8 @@ class logfile(object):
         self.closed = False
         self.filename = filename
         try:
-            self.file = open(expand_user_vars(self.filename), 'ab')
-        except IOError, (code, msg):
+            self.file = open(expand_user_vars(self.filename), 'a')
+        except IOError as msg:
             raise IOError('%s, opening file "%s"' % (msg, self.filename))
 
     def __del__(self):
@@ -208,9 +223,7 @@ def format_params(d, maskitems=('password', ), skipitems=()):
     '''Take a dictionary of parameters and return a string summary.
     '''
     s = ''
-    keys = d.keys()
-    keys.sort()
-    for key in keys:
+    for key in list(sorted(d.keys())):
         if key in skipitems:
             continue
         if s:
@@ -258,7 +271,7 @@ def is_maildir(d):
     return True
 
 #######################################
-def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
+def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0o600):
     '''Reliably deliver a mail message into a Maildir.  Uses Dan Bernstein's
     documented rules for maildir delivery, and the updated naming convention
     for new files (modern delivery identifiers).  See
@@ -274,7 +287,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
 
     info = {
         'deliverycount' : dcount,
-        'hostname' : hostname.split('.')[0].replace('/', '\\057').replace(
+        'hostname' : hostname.replace('/', '\\057').replace(
             ':', '\\072'),
         'pid' : os.getpid(),
     }
@@ -288,13 +301,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
         info['unique'] = 'M%(usecs)dP%(pid)s' % info
         if info['deliverycount'] is not None:
             info['unique'] += 'Q%(deliverycount)s' % info
-        try:
-            info['unique'] += 'R%s' % ''.join(
-                ['%02x' % ord(char)
-                 for char in open('/dev/urandom', 'rb').read(8)]
-            )
-        except StandardError:
-            pass
+        info['unique'] += 'R%s' % secrets.token_hex(8)
 
         filename = '%(secs)s.%(unique)s.%(hostname)s' % info
         fname_tmp = os.path.join(dir_tmp, filename)
@@ -327,19 +334,25 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
 
     # Open file to write
     try:
-        f = safe_open(fname_tmp, 'wb', filemode)
+        if sys.version_info.major > 2:
+            f = safe_open(fname_tmp, 'bw', filemode)
+        else:
+            f = safe_open(fname_tmp, 'w', filemode)
         f.write(data)
         f.flush()
         os.fsync(f.fileno())
         f.close()
 
-    except IOError, o:
+    except IOError as o:
         signal.alarm(0)
         raise getmailDeliveryError('failure writing file %s (%s)'
                                    % (fname_tmp, o))
 
     # Move message file from Maildir/tmp to Maildir/new
     try:
+        # #https://pypi.org/project/getmail_shutils/#description
+        # #Version of getmail using shutil instead of os.link to allow it to be used with a shared destination folder in a VM.
+        # shutil.copyfile(fname_tmp, fname_new)
         os.link(fname_tmp, fname_new)
         os.unlink(fname_tmp)
 
@@ -349,7 +362,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
             os.unlink(fname_tmp)
         except KeyboardInterrupt:
             raise
-        except StandardError:
+        except Exception:
             pass
         raise getmailDeliveryError('failure renaming "%s" to "%s"'
                                    % (fname_tmp, fname_new))
@@ -370,10 +383,12 @@ def mbox_from_escape(s):
 #######################################
 def address_no_brackets(addr):
     '''Strip surrounding <> on an email address, if present.'''
-    if addr.startswith('<') and addr.endswith('>'):
-        return addr[1:-1]
+    straddr = str(addr)
+    splitaddr = straddr.split('<')
+    if len(splitaddr) > 1:
+        return splitaddr[1].split('>')[0]
     else:
-        return addr
+        return straddr
 
 #######################################
 def eval_bool(s):
@@ -391,30 +406,30 @@ def eval_bool(s):
 def gid_of_uid(uid):
     try:
         return pwd.getpwuid(uid).pw_gid
-    except KeyError, o:
+    except KeyError as o:
         raise getmailConfigurationError('no such specified uid (%s)' % o)
 
 #######################################
 def uid_of_user(user):
     try:
         return pwd.getpwnam(user).pw_uid
-    except KeyError, o:
+    except KeyError as o:
         raise getmailConfigurationError('no such specified user (%s)' % o)
 
 #######################################
-def change_usergroup(logger=None, user=None, _group=None):
+def change_usergroup(logger=None, user=None, group=None):
     '''
     Change the current effective GID and UID to those specified by user and
-    _group.
+    group.
     '''
     uid = None
     gid = None
-    if _group:
+    if group:
         if logger:
-            logger.debug('Getting GID for specified group %s\n' % _group)
+            logger.debug('Getting GID for specified group %s\n' % group)
         try:
-            gid = grp.getgrnam(_group).gr_gid
-        except KeyError, o:
+            gid = grp.getgrnam(group).gr_gid
+        except KeyError as o:
             raise getmailConfigurationError('no such specified group (%s)' % o)
     if user:
         if logger:
@@ -423,6 +438,7 @@ def change_usergroup(logger=None, user=None, _group=None):
 
     change_uidgid(logger, uid, gid)
 
+
 #######################################
 def change_uidgid(logger=None, uid=None, gid=None):
     '''
@@ -440,32 +456,10 @@ def change_uidgid(logger=None, uid=None, gid=None):
                 if logger:
                     logger.debug('Setting euid to %d\n' % uid)
                 os.setreuid(uid, uid)
-    except OSError, o:
+    except OSError as o:
         raise getmailDeliveryError('change UID/GID to %s/%s failed (%s)'
                                    % (uid, gid, o))
 
-#######################################
-def decode_crappy_text(s):
-    '''Take a line of text in arbitrary and possibly broken bytestring encoding
-    and return an ASCII or unicode version of it.
-    '''
-    # first, assume it was written in the encoding of the user's terminal
-    lang = os.environ.get('LANG')
-    if lang:
-        try:
-            (lang, encoding) = lang.split('.')
-            return s.decode(encoding)
-        except (UnicodeError, ValueError), o:
-            pass
-    # that failed; try well-formed in various common encodings next
-    for encoding in ('ascii', 'utf-8', 'latin-1', 'utf-16'):
-        try:
-            return s.decode(encoding)
-        except UnicodeError, o:
-            continue
-    # all failed - force it
-    return s.decode('utf-8', 'replace')
-    
 
 #######################################
 def format_header(name, line):
@@ -539,8 +533,7 @@ def check_ca_certs(conf):
         ca_certs = expand_user_vars(ca_certs)
         if ssl is None:
             raise getmailConfigurationError(
-                'specifying ca_certs not supported by this installation of '
-                'Python; requires Python 2.6'
+                'specifying ca_certs not supported by this Python'
             )
     if ca_certs and not os.path.isfile(ca_certs):
         raise getmailConfigurationError(
@@ -555,8 +548,7 @@ def check_ssl_version(conf):
         return None
     if ssl is None:
         raise getmailConfigurationError(
-            'specifying ssl_version not supported by this installation of '
-            'Python; requires Python 2.6'
+            'specifying ssl_version not supported by this Python'
         )
     def get_or_fail(version, symbol):
         if symbol is not None:
@@ -583,12 +575,11 @@ def check_ssl_version(conf):
 #######################################
 def check_ssl_fingerprints(conf):
     ssl_fingerprints = conf['ssl_fingerprints']
-    if ssl_fingerprints is ():
+    if not ssl_fingerprints:
         return ()
     if ssl is None or hashlib is None:
         raise getmailConfigurationError(
-            'specifying ssl_fingerprints not supported by this installation of '
-            'Python; requires Python 2.6'
+            'specifying ssl_fingerprints not supported by this Python'
         )
 
     normalized_fprs = []
@@ -607,10 +598,9 @@ def check_ssl_ciphers(conf):
     if ssl_ciphers:
         if sys.version_info < (2, 7, 0):
             raise getmailConfigurationError(
-                'specifying ssl_ciphers not supported by this installation of '
-                'Python; requires Python 2.7'
+                'specifying ssl_ciphers not supported by this Python'
             )
-        if re.search(r'[^a-zA-z0-9, :!\-+@]', ssl_ciphers):
+        if re.search(r'[^a-zA-z0-9, :!\-+@=]', ssl_ciphers):
             raise getmailConfigurationError(
                 'invalid character in ssl_ciphers'
             )
@@ -619,13 +609,16 @@ def check_ssl_ciphers(conf):
 #######################################
 keychain_password = None
 if os.name == 'posix':
-    if os.path.isfile(osx_keychain_binary):
+    if keyring:
+        def keychain_password(user, server, protocol, logger):
+            return keyring.get_password(server,user)
+    elif os.path.isfile(osx_keychain_binary):
         def keychain_password(user, server, protocol, logger):
             """Mac OSX: return a keychain password, if it exists.  Otherwise, return
-         
+
          None.
             """
-            # OSX protocol is not an arbitrary string; it's a code limited to 
+            # OSX protocol is not an arbitrary string; it's a code limited to
             # 4 case-sensitive chars, and only specific values.
             protocol = protocol.lower()
             if 'imap' in protocol:
@@ -635,14 +628,14 @@ if os.name == 'posix':
             else:
                 # This will break.
                 protocol = '????'
-            
+
             # wish we could pass along a comment to this thing for the user prompt
             cmd = "%s find-internet-password -g -a '%s' -s '%s' -r '%s'" % (
                 osx_keychain_binary, user, server, protocol
             )
-            (status, output) = commands.getstatusoutput(cmd)
+            (status, output) = subprocess.getstatusoutput(cmd)
             if status != os.EX_OK or not output:
-                logger.error('keychain command %s failed: %s %s' 
+                logger.error('keychain command %s failed: %s %s'
                              % (cmd, status, output))
                 return None
             password = None
@@ -656,7 +649,7 @@ if os.name == 'posix':
                         pw = pw[1:-1]
                     password = pw
             if password is None:
-                logger.debug('No keychain password found for %s %s %s'
+                logger.debug('No keychain password found for %s %s %s\n'
                              % (user, server, protocol))
             return password
     elif gnomekeyring:
@@ -674,15 +667,15 @@ if os.name == 'posix':
                     # authtype=None, port=0
                     user, None, server, None, protocol, None, 0
                 )
-                
+
                 #logger.trace('got keyring result %s' % str(secret))
             except gnomekeyring.NoMatchError:
-                logger.debug('gnome-keyring does not know password for %s %s %s'
+                logger.debug('gnome-keyring does not know password for %s %s %s\n'
                              % (user, server, protocol))
                 return None
 
             # secret looks like this:
-            # [{'protocol': 'imap', 'keyring': 'Default', 'server': 'gmail.com', 
+            # [{'protocol': 'imap', 'keyring': 'Default', 'server': 'gmail.com',
             #   'user': 'hiciu', 'item_id': 1L, 'password': 'kielbasa'}]
             if secret and 'password' in secret[0]:
                 return secret[0]['password']
@@ -693,7 +686,7 @@ if os.name == 'posix':
         # Fallthrough
 if keychain_password is None:
     def keychain_password(user, server, protocol, logger):
-        """Neither Mac OSX keychain or Gnome keyring available: always return 
+        """Neither Mac OSX keychain or Gnome keyring available: always return
         None.
         """
         return None
@@ -704,47 +697,10 @@ def get_password(label, user, server, protocol, logger):
     # try keychain/keyrings first, where available
     password = keychain_password(user, server, protocol, logger)
     if password:
-        logger.debug('using password from keychain/keyring')
+        logger.debug('using password from keychain/keyring\n')
     else:
         # no password found (or not on OSX), prompt in the usual way
-        password = getpass.getpass('Enter password for %s:  ' % label)
+        password = getpass.getpass('Enter password for %s: ' % label)
     return password
 
 
-#######################################
-def run_command(command, args):
-    # Simple subprocess wrapper for running a command and fetching its exit 
-    # status and output/stderr.
-    if args is None:
-        args = []
-    if type(args) == tuple:
-        args = list(args)
-
-    # Programmer sanity checks
-    assert type(command) in (str, unicode), (
-        'command is %s (%s)' % (command, type(command))
-    )
-    assert type(args) == list, (
-        'args is %s (%s)' % (args, type(args))
-    )
-    for arg in args:
-        assert type(arg) in (str, unicode), 'arg is %s (%s)' % (arg, type(arg))
-
-    stdout = tempfile.TemporaryFile()
-    stderr = tempfile.TemporaryFile()
-
-    cmd = [command] + args
-
-    try:
-        p = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
-    except OSError, o:
-        if o.errno == errno.ENOENT:
-            # no such file, command not found
-            raise getmailConfigurationError('Program "%s" not found' % command)
-        #else:
-        raise
-
-    rc = p.wait()
-    stdout.seek(0)
-    stderr.seek(0)
-    return (rc, stdout.read().strip(), stderr.read().strip())
